#include <ddraw.h>
#include <sys/timeb.h>
#include <stdio.h>
#include <stdlib.h>

#define RES_X 1024
#define RES_Y 768
#define RES_BITDEPTH 32

#define S 15

LPDIRECTDRAW7 m_lpDD;
LPDIRECTDRAWSURFACE7	m_lpDDSFront;
LPDIRECTDRAWSURFACE7	m_lpDDSBack;
LPDIRECTDRAWSURFACE7	m_lpDDSOff;

HRESULT CreateSurface(DWORD xs, DWORD ys, LPDIRECTDRAWSURFACE7 *surface)
{
	HRESULT dwResult;
	DDSURFACEDESC2 ddsd;

	ZeroMemory(&ddsd, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
	ddsd.dwWidth = xs;
	ddsd.dwHeight = ys;
	ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;

	dwResult = m_lpDD->CreateSurface(&ddsd, surface, NULL);
	return dwResult;
}

HRESULT InitDDraw( HWND hWnd )
{
	m_lpDD = NULL;
	m_lpDDSFront = m_lpDDSBack = NULL;

    HRESULT dwResult;

    dwResult = DirectDrawCreateEx(NULL, (void**)&m_lpDD, IID_IDirectDraw7, NULL);
    if(dwResult != DD_OK) {
//        TRACE("Cannot create direct draw object due %d\n", dwResult);
        return dwResult;
    }

    dwResult = m_lpDD->SetCooperativeLevel(hWnd, DDSCL_EXCLUSIVE|
                                                DDSCL_FULLSCREEN|
                                                DDSCL_ALLOWREBOOT);
    if(dwResult != DD_OK) {
//        TRACE("Cannot set cooperative level due %d\n", dwResult);
        return dwResult;
    }

    dwResult = m_lpDD->SetDisplayMode(RES_X,RES_Y, RES_BITDEPTH, 0, 0);
    if(dwResult != DD_OK) {
//        TRACE("Cannot set display mode due %d\n", dwResult);
        return dwResult;
    }

	DDSURFACEDESC2 ddsd;

	ZeroMemory(&ddsd, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	ddsd.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP | 
			  DDSCAPS_COMPLEX;
	ddsd.dwBackBufferCount = 1;

	dwResult = m_lpDD->CreateSurface(&ddsd, &m_lpDDSFront, NULL);
	if(dwResult != DD_OK) {
//		TRACE("Cannot create front buffer due %d\n", dwResult);
		return dwResult;
	} 

	DDSCAPS2 ddscaps;

    ZeroMemory(&ddscaps, sizeof(ddscaps));
    ddscaps.dwCaps = DDSCAPS_BACKBUFFER;

	dwResult = m_lpDDSFront->GetAttachedSurface(&ddscaps, &m_lpDDSBack);
    if(dwResult != DD_OK) {
//		TRACE("Cannot create back buffer due %d\n", dwResult);
		return dwResult;
	}

	dwResult = CreateSurface(2*S, 2*S, &m_lpDDSOff);
	if(dwResult != DD_OK) {
//		TRACE("Cannot create surface due %d\n", dwRet);
	}


	return dwResult;
}



VOID Cleanup()
{
	if(m_lpDD) {
		if(m_lpDDSOff)
			m_lpDDSOff->Release();
		if(m_lpDDSBack)
			m_lpDDSBack->Release();
		if(m_lpDDSFront)
			m_lpDDSFront->Release();		
		m_lpDDSFront = m_lpDDSBack = m_lpDDSOff = NULL;
		m_lpDD->RestoreDisplayMode();
		m_lpDD->Release();
		m_lpDD = NULL;
	}
}

double	fps	= 1000;
int		i	= 0;

struct _timeb start, stop;

HPEN	hp = CreatePen(PS_SOLID, 1, RGB(155,255,255));
HBRUSH 	hb = CreateSolidBrush(RGB(0, 0, 0));
HDC hdc;
char sfps[20];

struct bod {
	double	x,
			y,
			vx,
			vy;
};

int f = 0;

RECT obd = {0, 0, RES_X, RES_Y};

RECT rcOff = {0, 0, 2*S, 2*S-3};
RECT rcFront = {RES_X/2-2*S, RES_Y/2-2*S, RES_X/2+2*S, RES_Y/2+2*S};

class barva {
public:
	COLORREF rgb;
	int r, g, b;
	void set(COLORREF cr) {
		rgb = cr;
		r = GetRValue(rgb);
		g = GetGValue(rgb);
		b = GetBValue(rgb);
	}
};

barva bod1, bod2, bod3;
int r, g, b;

void Render()
{

	if (m_lpDDSOff->GetDC(&hdc) == DD_OK)
	{
		if (f<2) {
			FillRect(hdc, &rcOff, hb);
			f ++;
		}

		SetPixel(hdc, rand()*0.00006*S, 2*S-1, RGB(255, 255-rand()*0.004, 0));
		SetPixel(hdc, rand()*0.00006*S, 2*S-1, RGB(0, 0, 0));
		for (int x = 1; x < 2*S-1; x ++) {
			for (int y = 0; y < 2*S-1; y ++) {
				bod1.set(GetPixel(hdc, x-1, y+1));
				bod2.set(GetPixel(hdc, x+1, y+1));
				bod3.set(GetPixel(hdc, x, y+1));
				r = bod1.r + bod2.r + bod3.r;
				g = bod1.g + bod2.g + bod3.g;
				b = bod1.b + bod2.b + bod3.b;
				SetPixel(hdc, x, y, RGB(r*0.329, g*0.325, b*0.32));
			}
		}
		
		m_lpDDSOff->ReleaseDC(hdc);
	}

//	m_lpDDSBack->Blt(&rcFront, m_lpDDSOff, &rcOff, DDBLT_WAIT , NULL); 
	m_lpDDSBack->Blt(&obd, m_lpDDSOff, &rcOff, DDBLT_WAIT , NULL); 

	if (m_lpDDSBack->GetDC(&hdc) == DD_OK)
	{
//		FillRect(hdc, &obd, hb);

		sprintf(sfps, "FPS: %.1f", fps);
		SetBkColor( hdc, RGB( 0, 0, 0 ) );
		SetTextColor( hdc, RGB( 0, 0, 255 ) );
		TextOut( hdc, 0, 0, sfps, lstrlen(sfps) );

		m_lpDDSBack->ReleaseDC(hdc);
	}

	i ++;
	if (i > 50)
	{
		_ftime(&stop);
		fps = i/((stop.time - start.time) + 0.001*(stop.millitm - start.millitm));
		_ftime(&start);
		i = 0;
	}

	m_lpDDSFront->Flip(NULL, DDFLIP_WAIT);
}


LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    switch( msg )
    {
        case WM_DESTROY:
            PostQuitMessage( 0 );
            return 0;

        case WM_PAINT:
            ValidateRect( hWnd, NULL );
            return 0;
    }

    return DefWindowProc( hWnd, msg, wParam, lParam );
}



INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT )
{
    WNDCLASSEX wc = { sizeof(WNDCLASSEX), CS_CLASSDC, MsgProc, 0L, 0L, 
                      GetModuleHandle(NULL), NULL, NULL, NULL, NULL,
                      "Direct Draw Test", NULL };
    RegisterClassEx( &wc );

    HWND hWnd = CreateWindow( "Direct Draw Test", "Direct Draw Test 1", 
                              WS_EX_TOPMOST, 100, 100, 300, 300,
                              GetDesktopWindow(), NULL, wc.hInstance, NULL );
//WS_EX_TOPMOST
//WS_OVERLAPPEDWINDOW
    if( SUCCEEDED( InitDDraw( hWnd ) ) )
    { 
        ShowWindow( hWnd, SW_SHOWDEFAULT );
        UpdateWindow( hWnd );

        MSG msg;
		BOOL gm = TRUE;

		_ftime(&start);

		while ( gm ) {
			if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
			{
				if( gm = GetMessage( &msg, NULL, 0, 0 ) )
				{
					TranslateMessage( &msg );
					DispatchMessage( &msg );
				}
			}
			else Render();
		}
    }

    Cleanup();
    UnregisterClass( "Direct Draw Test", wc.hInstance );
    return 0;
}



